ManagerDict = dict(
	RYXB="人员性别",
	RYXM="人员姓名",
	RYXL="人员学历",
	CYNX="从业年限",
	RZRQ="入职日期",
	LRRQ="离任日期",
	RYZT="人员状态",
	JJDM="基金代码",
	GGRQ="公告日期",
	RYDM="人员代码",
)


FundInfoDict = dict(
	FGSX="风格属性",
	TZFG="投资风格",
	JJJC="基金简称",
	JJXZ="基金性质",
	CLRQ="成立日期",
	JJFL="基金分类",
	GGRQ="公告日期",
	JJDM="基金代码",
	CPFL="产品分类",    # 1为固收 2为公募 4为私募
	ZZRQ="终止日期",
	JXFL=""
)


FundNetValueDict = dict(
	JJDM="基金代码",
	JZRQ="净值日期",
	LJJZ="累计净值",
	JJJZ="基金净值",
)


PublicFundFirstCategory = {
	"1": "股票型",
	"3": "混合型",
	"7": "货币型",
	"9": "QDII",
	"2": "债券型",
	"f": "FOF",
	"4": "另类投资"
}


PublicFundSecondCategory = {
	"13": "普通股票型",
	"14": "股票型",
	"15": "增强指数型",
	"35": "灵活配置型",
	"16": "被动指数型",
	"37": "偏股混合型",
	"36": "混合型",
	"38": "偏债混合型",
	"91": "股票型QDII",
	"93": "混合型QDII",
	"f3": "债券型FOF",
	"94": "债券型QDII",
	"42": "商品型",
	"f1": "股票型FOF",
	"95": "另类型QDII",
	"41": "股票多空",
	"21": "被动指数型债券",
	"f4": "货币型FOF",
	"23": "混合债券型一级",
	"22": "短期纯债型",
	"25": "增强指数型债券",
	"26": "债券型",
	"43": "REITs",
	"27": "中长期纯债型",
	"24": "混合债券型二级",
	"f2": "混合型FOF",
	"f5": "另类投资FOF"
}




