# -*- coding:utf-8 -*- 
import sys
PY3 = (sys.version_info[0] >= 3)
HTTP_OK = 200
HTTP_AUTHORIZATION_ERROR = 401
HTTP_URL = 'data.howbuy.com'
HTTP_PORT = 443
TOKEN_F_P = 'tk.csv'
TOKEN_ERR_MSG = '请设置hbshare的token凭证码，如果没有请联系好买FDC注册申请'

ENV_F_P = 'env.csv'
ENV_ERR_MSG = '调用接口涉及环境参数，请设置hbshare的环境参数'
ENV_PROD = 'prod'

